#!/usr/bin/bash
# build_ipa_server.sh - Build a standard IPA Server for nfs related tests

Usage() {
cat <<END
build_ipa_server.sh: Build a standard IPA Server for nfs related tests
Usage: build_ipa_server.sh [-p|--password password]
                           [-c|--cleanup] [--config_krb]

        -h|--help                        # Print this help

        [Basic Function: Config current client as an IPA Domain Server]
        -p|--passwd                      # Specify a domain super password instead of default
        -c|--cleanup                     # Uninstall and downgrade current IPA Server

        [Extra Function: Config extra roles (secure NFS Client..) after integration]
        --config_krb                     # Config current client as a Secure NFS client

END
}

infoecho() { echo -e "\n${P:-DO ==>}" "\E[1;34m" "$@" "\E[0m"; }
errecho()  { echo -e "\n${P:-DO ==>}" "\E[31m" "$@" "\E[0m"; }

# PART: [Dependency] Specify all dependencies during installation

# Specify key packages for "Building IPA Server"
export pkgs="ipa-server ipa-server-dns"

# Specify key parameters during "Building IPA Server"

# WARN: An unique IPA Domain name is necessary for a new installation
#
# For beaker baesd auto tests, a unique JOBID can be obtained from each beaker job
# For other tests, default JOBID will be assigned as a timestamp like: 2017.may09.081201

test -z $JOBID && \
	export JOBID="$(date +%Y.%h%d.%H%M | tr A-Z a-z)"

export IPA_DOMAIN_NAME=$(echo ipa-domain.job-$JOBID | sed 's/[^0-9a-z.]//g')

export IPA_DOMAIN_SUPERPW_COMMON=fsqe2015!
export IPA_DOMAIN_SUPERPW=$IPA_DOMAIN_SUPERPW_COMMON

export IPA_SERVER_SHORT_NAME=$(hostname|cut -d"." -f1)
export IPA_SERVER_FQDN=${IPA_SERVER_SHORT_NAME}.${IPA_DOMAIN_NAME}

export IPA_SERVER_IP=$(getDefaultIp)
export IPA_SERVER_IP_DOMAIN=$(echo $IPA_SERVER_IP | awk -F"." '{print $3"."$2"."$1".in-addr.arpa"}')
export IPA_SERVER_IP_MEMBER=$(echo $IPA_SERVER_IP | awk -F"." '{print $4}')

# Specify functions
export cleanup=no;
export config_krb=no;

while [ -n "$1" ]; do
        case "$1" in
        -p|--passwd)  IPA_DOMAIN_SUPERPW="$2";shift 2;;
        -h|--help)    Usage;exit 0;;
        -c|--cleanup) cleanup="yes";shift 1;;
        --config_krb) config_krb="yes";shift 1;;
        *) break;;
        esac
done

# [Basic Function] Uninstall and downgrade IPA Server

test "$cleanup" == "yes" && {
	kinit admin <<< $IPA_DOMAIN_SUPERPW
	infoecho "Start cleanup: Uninstall and downgrade IPA Server"
	if ipa-server-install --uninstall --unattend; then
		infoecho "IPA Server Uninstall Success..."
		exit 0
	else
		errecho "IPA Server Uninstall Failed..."
		exit 1
	fi
}

# [Basic Function] Config current node as an IPA Server

infoecho "\n\nStart to configure current node as IPA Server for domain $IPA_DOMAIN_NAME\n"

infoecho "1. Close the firewall and block NetworkManager..."
[ -f /etc/init.d/iptables ] && service iptables stop
which systemctl &>/dev/null && systemctl stop firewalld

# Block NetworkManager to ensure change in resolv.conf is effective
service NetworkManager status &>/dev/null && service NetworkManager stop &>/dev/null
service NetworkManager status &>/dev/null || service network start &>/dev/null

infoecho "2. Make sure necessary packages are installed..."
rpm -q $pkgs || {
	echo "Typically, there will be more then 200 pkgs in dependency list (RHEL-7.3), keep silence during installation"
	if yum -y install $pkgs &> /tmp/yum-log-$JOBID; then
		echo "Installation Finished for package $pkgs"
		rpm -q $pkgs && rm -rf /tmp/yum-log-$JOBID
	else
		echo "Installation Failed for package $pkgs, abort";
		echo "Check yum installation log to obtain more information:"
		cat --number /tmp/yum-log-$JOBID && rm -rf /tmp/yum-log-$JOBID
		exit 1
fi
}

infoecho "3. Change hostname to integrate into IPA DNS Domain..."
hostnamectl set-hostname $IPA_SERVER_FQDN

infoecho "4. Build up IPA Server..."
ipa-server-install --realm=$IPA_DOMAIN_NAME --domain=$IPA_DOMAIN_NAME \
	--ds-password=$IPA_DOMAIN_SUPERPW --master-password=$IPA_DOMAIN_SUPERPW \
	--admin-password=$IPA_DOMAIN_SUPERPW --hostname=$IPA_SERVER_FQDN \
	--setup-dns --no-forwarders --unattended

infoecho "5. Fetch krbtgt for domain super user..."
kinit admin <<< $IPA_DOMAIN_SUPERPW
klist | grep krbtgt

infoecho "6. Add DNS records for IPA Server..."
ipa dnszone-add $IPA_SERVER_IP_DOMAIN
ipa dnsrecord-add $IPA_DOMAIN_NAME $IPA_SERVER_SHORT_NAME --a-rec $IPA_SERVER_IP
ipa dnsrecord-add $IPA_SERVER_IP_DOMAIN $IPA_SERVER_IP_MEMBER --ptr-rec "${IPA_SERVER_FQDN}."

infoecho "7. Check IPA Server Status..."
if ipactl status 2>/dev/null | grep -v FAILED; then
	infoecho "IPA Server Configuration Success..."
else
	errecho "IPA Server Configuration Failed..."
	exit 1
fi

# PART: [Extra Functions] Config current client as a Secure NFS client

if [ "$config_krb" == "yes" ]; then
	infoecho "Config current client as a Secure NFS client..."
	for serviceType in root nfs; do
		# Add principal to this node by domain super user admin
		ipa service-add $serviceType/$(hostname)
		# Fetch principal from remote server to local keytab
		ipa-getkeytab -s $IPA_SERVER_FQDN -p $serviceType/$(hostname) -k /etc/krb5.keytab
		# Check whether local IPA client could read target principal from keytab
		ipa service-show $serviceType/$(hostname)
		if ! klist -kte | grep "$serviceType/$(hostname)"; then
			errecho "Add principle $serviceType/$(hostname) Failed..."
			exit 1
		fi
	done
	infoecho "Add principle (nfs|root)/$(hostname) Success..."
	exit 0
fi

exit 0
